using Microsoft.AspNetCore.Mvc;
using PharmacistApp.Models;
using System.Diagnostics;
using VetClinicContracts.BindingModels;
using VetClinicContracts.ViewModels;
using VetClinicDataBaseImplement.Models;

namespace PharmacistApp.Controllers
{
	public class HomeController : Controller
	{
		private readonly ILogger<HomeController> _logger;

		public HomeController(ILogger<HomeController> logger)
		{
			_logger = logger;
		}

		public IActionResult Index()
		{
			if (APIPharmacist.Pharmacist == null)
			{
				return Redirect("~/Home/Enter");
			}
			return
View(APIPharmacist.GetRequest<List<MedicineViewModel>>($"api/medicine/getmedicines?pharmacistid={
	APIPharmacist.Pharmacist.Id}"));

		}

		[HttpGet]
		public IActionResult Privacy()
		{
			if (APIPharmacist.Pharmacist == null)
			{
				return Redirect("~/Home/Enter");
			}
			return View(APIPharmacist.Pharmacist);
		}

		[HttpPost]
		public void Privacy(string email, string password, string fio)
		{
			if (APIPharmacist.Pharmacist == null)
			{
				throw new Exception("   ?    ");
			}
			if (string.IsNullOrEmpty(email) ||
			string.IsNullOrEmpty(password) || string.IsNullOrEmpty(fio))
			{
				throw new Exception(" email,   ");
			}
			APIPharmacist.PostRequest("api/pharmacist/updatedata", new
			PharmacistBindingModel
			{
				Id = APIPharmacist.Pharmacist.Id,
				PharmacistFIO = fio,
				Email = email,
				Password = password
			});
			APIPharmacist.Pharmacist.PharmacistFIO = fio;
			APIPharmacist.Pharmacist.Email = email;
			APIPharmacist.Pharmacist.Password = password;
			Response.Redirect("Index");
		}

		[HttpGet]
		public IActionResult Enter()
		{
			return View();
		}
		[HttpPost]
		public void Enter(string email, string password)
		{
			if (string.IsNullOrEmpty(email) ||
			string.IsNullOrEmpty(password))
			{
				throw new Exception(" email  ");
			}
			APIPharmacist.Pharmacist =
			APIPharmacist.GetRequest<PharmacistViewModel>($"api/pharmacist/login?email={email}&password={password}");
			if (APIPharmacist.Pharmacist == null)
			{
				throw new Exception(" /");
			}
			Response.Redirect("Index");
		}
		[HttpGet]
		public IActionResult Register()
		{
			return View();
		}
		[HttpPost]
		public void Register(string email, string password, string fio)
		{
			if (string.IsNullOrEmpty(email) ||
			string.IsNullOrEmpty(password) || string.IsNullOrEmpty(fio))
			{
				throw new Exception(" ,   ");
			}
			APIPharmacist.PostRequest("api/pharmacist/register", new
			PharmacistBindingModel
			{
				PharmacistFIO = fio,
				Email = email,
				Password = password
			});
			Response.Redirect("Enter");
			return;
		}

		public IActionResult CreateMedicine()
		{
			if (APIPharmacist.Pharmacist == null)
			{
				return Redirect("~/Home/Enter");
			}
			return View();
		}



		[HttpPost]
		public void CreateMedicine(string name, int price)
		{
			if (APIPharmacist.Pharmacist == null)
			{
				throw new Exception("   ?    ");
			}
			if (string.IsNullOrEmpty(name) || price <= 0)
			{
				throw new Exception("   ");
			}
			APIPharmacist.PostRequest("api/medicine/createmedicine", new MedicineBindingModel
			{
				MedicineName = name,
				Price = price,
				PharmacistId = APIPharmacist.Pharmacist.Id
			}) ;
			Response.Redirect("Index");
		}

		public IActionResult DeleteMedicine()
		{
			if (APIPharmacist.Pharmacist == null)
			{
				return Redirect("~/Home/Enter");
			}
			ViewBag.Medicines = APIPharmacist.GetRequest<List<MedicineViewModel>>("api/medicine/getmedicines");
			return View();
		}

		[HttpPost]
		public void DeleteMedicine(int medicine)
		{
			if (APIPharmacist.Pharmacist == null)
			{
				throw new Exception("   ?    ");
			}
			APIPharmacist.PostRequest("api/medicine/deletemedicine", new MedicineBindingModel
			{
				Id = medicine
			});
			Response.Redirect("Index");
		}

		public IActionResult UpdateMedicine()
		{
			if (APIPharmacist.Pharmacist == null)
			{
				return Redirect("~/Home/Enter");
			}
			ViewBag.Medicines = APIPharmacist.GetRequest<List<MedicineViewModel>>("api/medicine/getmedicines");
			return View();
		}

		[HttpPost]
		public void UpdateMedicine(int medicine, string name, int price)
		{
			if (APIPharmacist.Pharmacist == null)
			{
				throw new Exception("   ?    ");
			}
			if (string.IsNullOrEmpty(name) || price <= 0)
			{
				throw new Exception("   ");
			}
			APIPharmacist.PostRequest("api/medicine/updatemedicine", new MedicineBindingModel
			{
				Id = medicine,
				MedicineName = name,
				Price = price,
			});
			Response.Redirect("Index");
		}

		[ResponseCache(Duration = 0, Location = ResponseCacheLocation.None, NoStore = true)]
		public IActionResult Error()
		{
			return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
		}

		[HttpGet]
		public Tuple<MedicineViewModel, string>? GetMedicine(int medicineId)
		{
			if (APIPharmacist.Pharmacist == null)
			{
				throw new Exception("   ?    ");
			}
			var result = APIPharmacist.GetRequest<Tuple<MedicineViewModel, List<string>>>($"api/medicine/getmedicine?medicineid={medicineId}");
			if (result == null)
			{
				return default;
			}
			string table = "";
			result.Item1.MedicineAnimals.Clear();
			for (int i = 0; i < result.Item2.Count; i++)
			{
				var animal = result.Item2[i];
				table += "<tr>";
				table += $"<td>{animal}</td>";
				table += "</tr>";
			}
			return Tuple.Create(result.Item1, table);
		}
	}
}
